##################################################################################################
# Copyright 2009 Seapine Software, Inc. All rights reserved.
#
# IMPORTANT:  This Software is supplied to you by Seapine Software, Inc. ("Seapine")
# in consideration of your agreement to the following terms, and your use, installation,
# modification or redistribution of this Software constitutes acceptance of these terms.
# If you do not agree with these terms, please do not use, install, modify or redistribute
# this Software.
#
# In consideration of your agreement to abide by the following terms, and subject to these
# terms, Seapine grants you a personal, non-exclusive license, under Seapine copyrights in
# this original Seapine software (the "Software"), to use, reproduce, modify and redistribute
# the Software, with or without modifications, in source and/or binary forms; provided that
# if you redistribute the Software in its entirety and without modifications, you must retain
# this notice and the following text and disclaimers in all such redistributions of the Software.
# Neither the name, trademarks, service marks or logos of Seapine, Inc. may be used to endorse
# or promote products derived from the Software without specific prior written permission from
# Seapine.  Except as expressly stated in this notice, no other rights or licenses, express or
# implied, are granted by Seapine herein, including but not limited to any patent rights that
# may be infringed by your derivative works or by other works in which the Software may be
# incorporated.
#
# The Software is provided by Seapine on an "AS IS" basis.  SEAPINE MAKES NO WARRANTIES,
# EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION THE IMPLIED WARRANTIES OF NON-INFRINGEMENT,
# MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE, REGARDING THE SOFTWARE OR ITS USE AND
# OPERATION ALONE OR IN COMBINATION WITH YOUR PRODUCTS.
#
# IN NO EVENT SHALL SEAPINE BE LIABLE FOR ANY SPECIAL, INDIRECT, INCIDENTAL OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) ARISING IN ANY WAY OUT OF THE USE,
# REPRODUCTION, MODIFICATION AND/OR DISTRIBUTION OF THE SOFTWARE, HOWEVER CAUSED AND WHETHER
# UNDER THEORY OF CONTRACT, TORT (INCLUDING NEGLIGENCE), STRICT LIABILITY OR OTHERWISE, EVEN
# IF SEAPINE HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
##################################################################################################

import getopt
import httplib
import base64
import os
import StringIO
import sys
import urllib
import xml
from xml.dom import DOMImplementation, DocumentType
from xml.dom.ext import PrettyPrint
import xml.dom.minidom

# cmd-line syntax: <project name> <username> <password> <report name> <save path>

class Usage(Exception):
    def __init__(self, msg):
        self.msg = msg

# We need to pass this information in various formats, so break up the URL as needed.
c_soapaddr = "127.0.0.1"
c_soapport = 80
c_soapcgi = "http://" + c_soapaddr + ":" + str(c_soapport) + "/cgi-bin/ttsoapcgi.exe"

def runReport(p_session, p_report, p_savepath):
   dom = DOMImplementation.getDOMImplementation ()
   typ = xml.dom.DocumentType.DocumentType ("Envelope", [], [], "", "")
   # this is not the namespace required by soap 1.2
   # but the service requires this namespace (from error message)
   doc = dom.createDocument ("http://schemas.xmlsoap.org/soap/envelope/", "Envelope", typ)

   env = doc.getElementsByTagName ("Envelope")[0]
   bdy = doc.createElement ("Body")
   env.appendChild (bdy)

   msg = doc.createElementNS ("urn:testtrack-interface", "getReportRunResultsByName")
   bdy.appendChild (msg)

   eleCookie = doc.createElement("cookie")
   eleCookie.setAttributeNode( doc.createAttribute("xsi:type"))
   eleCookie.setAttribute("xsi:type", "xsd:int")
   msg.appendChild(eleCookie)
   eleCookie.appendChild(doc.createTextNode(str(p_session)))

   eleName = doc.createElement("name")
   eleName.setAttributeNode( doc.createAttribute("xsi:type"))
   eleName.setAttribute("xsi:type", "xsd:string")
   msg.appendChild(eleName)
   eleName.appendChild(doc.createTextNode(str(p_report)))

   # Let's run the report!
   buffer = StringIO.StringIO()
   PrettyPrint(doc, buffer)
   request = buffer.getvalue()
   # print request #harpm_debug

   httpsvr = httplib.HTTPConnection(c_soapaddr, c_soapport)
   httpsvr.putrequest("POST", c_soapcgi)
   httpsvr.putheader("Content-Type", "text/xml")
   httpsvr.putheader("Content-Length", str(len(request)))
   httpsvr.endheaders()
   httpsvr.send(request)

   response = httpsvr.getresponse()
   print "getReportRunResultsByName", response.status, response.reason

   dom = xml.dom.minidom.parseString(response.read())
   records = dom.getElementsByTagName("item")

   i = 0
   # Save any addtl files   sent back
   for report in records:
      if i == 0: # this is the report itself, so save it w/ the report name
         filepath = os.path.join(p_savepath, urllib.quote(p_report) + '.html') # make the filename html-safe.
      else: # supporting files, save as-are
         file = report.getElementsByTagName("m-strFileName")[0].childNodes[0].data
         filepath = os.path.join(p_savepath, file)

      # print filepath #harpm_debug

      FILE = open(filepath, "wb") # write as binary
      FILE.write( base64.b64decode( report.getElementsByTagName("m-pFileData")[0].childNodes[0].data)) # decode SOAP results
      FILE.close()
      i=i+1

def getProjectList(p_user, p_pwd):
   dom = DOMImplementation.getDOMImplementation ()
   typ = xml.dom.DocumentType.DocumentType ("Envelope", [], [], "", "")
   # this is not the namespace required by soap 1.2
   # but the service requires this namespace (from error message)
   doc = dom.createDocument ("http://schemas.xmlsoap.org/soap/envelope/", "Envelope", typ)

   env = doc.getElementsByTagName ("Envelope")[0]
   bdy = doc.createElement ("Body")
   env.appendChild (bdy)

   msg = doc.createElementNS ("urn:testtrack-interface", "getProjectList")
   bdy.appendChild (msg)

   ele = doc.createElement("username")
   ele.setAttributeNode( doc.createAttribute("xsi:type"))
   ele.setAttribute("xsi:type", "xsd:string")
   msg.appendChild(ele)
   ele.appendChild(doc.createTextNode(str(p_user)))

   ele = doc.createElement("password")
   ele.setAttributeNode( doc.createAttribute("xsi:type"))
   ele.setAttribute("xsi:type", "xsd:string")
   msg.appendChild(ele)
   ele.appendChild(doc.createTextNode(str(p_pwd)))

   buffer = StringIO.StringIO()
   PrettyPrint(doc, buffer)
   request = buffer.getvalue()
   # print request #harpm_debug

   httpsvr = httplib.HTTPConnection(c_soapaddr, c_soapport)
   httpsvr.putrequest("POST", c_soapcgi)
   httpsvr.putheader("Content-Type", "text/xml")
   httpsvr.putheader("Content-Length", str(len(request)))
   httpsvr.endheaders()
   httpsvr.send(request)

   response = httpsvr.getresponse()
   print "getProjectList", response.status, response.reason
   return response.read()

def validProjectLicenseTypes(p_project, p_user, p_pwd):
   prjResult = []
   dom = xml.dom.minidom.parseString(getProjectList(p_user, p_pwd))
   aprojects = dom.getElementsByTagName("item")

   for prj in aprojects:
      name = prj.getElementsByTagName("name")[0].childNodes[0].data
      if name == p_project:
         strOpts = prj.getElementsByTagName("options")[0].toxml()
         if strOpts.find("TestTrack Pro") != -1:
            prjResult.append("ttp")
         if strOpts.find("TestTrack TCM") != -1:
            prjResult.append("tcm")
         # prjResult = prj #.toxml() #.replace("<item>", "").replace("</item>","")
         # prjResult = prj.getElementsByTagName("options")[0].toxml()
         break

   # dom.unlink()
   return prjResult

def login(p_project, p_user, p_pwd):
   dom = DOMImplementation.getDOMImplementation ()
   typ = xml.dom.DocumentType.DocumentType ("Envelope", [], [], "", "")
   # this is not the namespace required by soap 1.2
   # but the service requires this namespace (from error message)
   doc = dom.createDocument ("http://schemas.xmlsoap.org/soap/envelope/", "Envelope", typ)

   env = doc.getElementsByTagName ("Envelope")[0]
   bdy = doc.createElement ("Body")
   env.appendChild (bdy)

   msg = doc.createElementNS ("urn:testtrack-interface", "ProjectLogon")
   bdy.appendChild (msg)

   eleProject = doc.createElement("pProj")
   eleProject.setAttributeNode( doc.createAttribute("href"))
   eleProject.setAttribute("xsi:type", "tns:CProject")
   msg.appendChild(eleProject)

   # Add database name
   eleDb = doc.createElement("database")
   eleDb.setAttribute("xsi:type", "tns:CDatabase")
   eleProject.appendChild(eleDb)
   eleName = doc.createElement("name")
   eleName.setAttribute("xsi:type", "xsd:string")
   eleName.appendChild(doc.createTextNode(str(p_project)))
   eleDb.appendChild(eleName)

   # Be dynamic on license request
   aprjLicenseTypes = validProjectLicenseTypes(p_project, p_user, p_pwd)

   # Add our login options, essentially which licenses we want to pull.
   eleOptions = doc.createElement("options")
   eleOptions.setAttribute("xsi:type", "tns:CProjectDataOption[" + str(len(aprjLicenseTypes)) + "]")
   eleProject.appendChild(eleOptions)

   for lt in aprjLicenseTypes:
      eleOption = doc.createElement("tns:CProjectDataOption")
      eleOption.setAttribute("xsi:type", "tns:CProjectDataOption")
      eleOptions.appendChild(eleOption)
      eleName = doc.createElement("name")
      eleName.setAttribute("xsi:type", "xsd:string")
      if lt == "ttp":
         eleName.appendChild(doc.createTextNode("TestTrack Pro"))
      if lt == "tcm":
         eleName.appendChild(doc.createTextNode("TestTrack Pro"))
      eleOption.appendChild(eleName)

   # Set username
   eleUser = doc.createElement("username")
   eleUser.setAttributeNode( doc.createAttribute("xsi:type"))
   eleUser.setAttribute("xsi:type", "xsd:string")
   msg.appendChild(eleUser)
   eleUser.appendChild(doc.createTextNode(str(p_user)))

   # Set password
   elePwd = doc.createElement("password")
   elePwd.setAttributeNode( doc.createAttribute("xsi:type"))
   elePwd.setAttribute("xsi:type", "xsd:string")
   msg.appendChild(elePwd)
   elePwd.appendChild(doc.createTextNode(str(p_pwd)))

   buffer = StringIO.StringIO()
   PrettyPrint(doc, buffer)
   request = buffer.getvalue()
   # print request #harpm_debug

   httpsvr = httplib.HTTPConnection(c_soapaddr, c_soapport)
   httpsvr.putrequest("POST", c_soapcgi)
   httpsvr.putheader("Content-Type", "text/xml")
   httpsvr.putheader("Content-Length", str(len(request)))
   httpsvr.endheaders()
   httpsvr.send(request)

   response = httpsvr.getresponse()
   session = response.read()
   print "DatabaseLogon", response.status, response.reason

   return session[session.find("<Cookie>")+8:session.find("</Cookie>")]

def logout(p_session):
   dom = DOMImplementation.getDOMImplementation ()
   typ = xml.dom.DocumentType.DocumentType ("Envelope", [], [], "", "")
   # this is not the namespace required by soap 1.2
   # but the service requires this namespace (from error message)
   doc = dom.createDocument ("http://schemas.xmlsoap.org/soap/envelope/", "Envelope", typ)

   env = doc.getElementsByTagName ("Envelope")[0]
   bdy = doc.createElement ("Body")
   env.appendChild (bdy)

   msg = doc.createElementNS ("urn:testtrack-interface", "DatabaseLogoff")
   bdy.appendChild (msg)

   eleCookie = doc.createElement("cookie")
   eleCookie.setAttributeNode( doc.createAttribute("xsi:type"))
   eleCookie.setAttribute("xsi:type", "xsd:int")
   msg.appendChild(eleCookie)
   eleCookie.appendChild(doc.createTextNode(str(p_session)))

   # Let's lock the defect for editing!
   buffer = StringIO.StringIO()
   PrettyPrint(doc, buffer)
   request = buffer.getvalue()

   httpsvr = httplib.HTTPConnection(c_soapaddr, c_soapport)
   httpsvr.putrequest("POST", c_soapcgi)
   httpsvr.putheader("Content-Type", "text/xml")
   httpsvr.putheader("Content-Length", str(len(request)))
   httpsvr.endheaders()
   httpsvr.send(request)

   response = httpsvr.getresponse()
   print "DatabaseLogoff", response.status, response.reason

def main(argv=None):
   if argv is None:
      argv = sys.argv
   try:
      try:
         opts, args = getopt.getopt(argv[1:], "h", ["help"])

         # check that we're being called correctly!
         if len(args) != 5:
            print "\n\nThe script requires the following parameters (in this order):"
            print "\n\tProject: TestTrack project name (plain-text)"
            print "\tUsername: TestTrack username (plain-text)"
            print "\tPassword: TestTrack password (plain-text)"
            print "\tReport: TestTrack report to run"
            print "\tPath: Path to save the report's resulting run data to"
            sys.exit(2)

         try:
            session = 0
            session = login(args[0], args[1], args[2])
            runReport(session, args[3], args[4])
         finally:
            logout(session)

      except getopt.error, msg:
         raise Usage(msg)
         # more code, unchanged
   except Usage, err:
      print >>sys.stderr, err.msg
      print >>sys.stderr, "for help use --help"
      return 2

if __name__ == "__main__":
   sys.exit(main())
